KeyMaterial

Struct KeyMaterial 

pub struct KeyMaterial<const KEY_LEN: usize> { /* private fields */ }
A wrapper for holding bytes-like key material (symmetric keys or seeds) which aims to apply a strict typing system to prevent many kinds of mis-use mistakes. The capacity of the internal buffer can be set at compile-time via the <KEY_LEN> param.

Implementations

Source

impl<const KEY_LEN: usize> KeyMaterial<KEY_LEN>

pub fn new() -> Self

Creates a new empty instance (key_len = 0, key_type = Zeroized). If you want a properly populated instance, use KeyMaterial::from_rng.

pub fn from_rng(rng: &mut impl RNG) -> Result<Self, KeyMaterialError>

Creates a new instance of KeyMaterial containing random bytes from the provided random number generator.

pub fn from_bytes(source: &[u8]) -> Result<Self, KeyMaterialError>

Constructor. Loads the provided data into a new KeyMaterial of type KeyType::Unknown. It will detect if you give it all-zero source data and set the key type to KeyType::Zeroized instead.

pub fn from_bytes_as_type( source: &[u8], key_type: KeyType, ) -> Result<Self, KeyMaterialError>

Constructor. Loads the provided data into a new KeyMaterial of the specified type. This is discouraged unless the caller knows the provenance of the data, such as loading it from a cryptographic private key file. It will detect if you give it all-zero source data and set the key type to KeyType::Zeroized instead.

Will set the SecurityStrength automatically according to the following rules:

pub fn from_key(other: &impl KeyMaterialTrait) -> Result<Self, KeyMaterialError>

Copy constructor